"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.CardLayout = void 0;
var _react = _interopRequireDefault(require("react"));
var _reactNative = require("react-native");
var _models = require("./models");
var _CardFieldNumber = require("./CardFieldNumber");
var _CardFieldExpMm = require("./CardFieldExpMm");
var _CardFieldExpYy = require("./CardFieldExpYy");
var _CardFieldCvv = require("./CardFieldCvv");
var _CvvUtils = require("./CvvUtils");
function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }
class CardLayout extends _react.default.Component {
  constructor(props) {
    super(props);
    if (!props.inputNumber || typeof props.inputNumber !== 'function') {
      throw new Error('inputNumber prop is required');
    }
    if (!props.inputExpMm || typeof props.inputExpMm !== 'function') {
      throw new Error('inputExpMm prop is required');
    }
    if (!props.inputExpYy || typeof props.inputExpYy !== 'function') {
      throw new Error('inputExpYy prop is required');
    }
    if (!props.inputCvv || typeof props.inputCvv !== 'function') {
      throw new Error('inputCvv prop is required');
    }
    this._displayedCard = undefined;
  }
  getCard = () => {
    if (this._displayedCard) {
      return null;
    }
    const cardNumber = this._inputNumber._getText();
    const expMm = this._inputExpMm._getText();
    const expYy = this._inputExpYy._getText();
    const expCvv = this._inputCvv._getText();
    const card = new _models.Card();
    const cardPrivate = card;
    cardPrivate.__getCardNumber__ = () => {
      return cardNumber;
    };
    cardPrivate.__getExpYy__ = () => {
      try {
        return Number(expYy);
      } catch (e) {
        return 0;
      }
    };
    cardPrivate.__getExpMm__ = () => {
      try {
        return Number(expMm);
      } catch (e) {
        return 0;
      }
    };
    cardPrivate.__getCvv__ = () => {
      return expCvv;
    };
    return card;
  };
  showCard = card => {
    let enabled = true;
    if (card) {
      if (!(card instanceof _models.Card)) {
        throw new Error('Invalid card to show');
      }
      const cardPrivate = card;
      this._inputNumber._setText(cardPrivate.__getCardNumber__());
      this._inputExpMm._setText(String(cardPrivate.__getExpYy__()));
      this._inputExpYy._setText(String(cardPrivate.__getCardNumber__()));
      this._displayedCard = cardPrivate;
    } else {
      enabled = false;
      this._inputNumber._setText('');
      this._inputExpMm._setText('');
      this._inputExpYy._setText('');
      this._displayedCard = undefined;
    }
    this._inputCvv._setText('');
    this._inputNumber._setEnable(enabled);
    this._inputExpMm._setEnable(enabled);
    this._inputExpYy._setEnable(enabled);
    this._inputCvv._setEnable(enabled);
  };
  test = () => {
    this._inputNumber._setText('4444555566661111');
    this._inputExpMm._setText('12');
    this._inputExpYy._setText('25');
    this._inputCvv._setText('111');
  };
  componentDidMount() {
    this._inputNumber = this._pullInput('inputNumber', _CardFieldNumber.CardFieldNumber);
    this._inputExpMm = this._pullInput('inputExpMm', _CardFieldExpMm.CardFieldExpMm);
    this._inputExpYy = this._pullInput('inputExpYy', _CardFieldExpYy.CardFieldExpYy);
    this._inputCvv = this._pullInput('inputCvv', _CardFieldCvv.CardFieldCvv);
    this._inputNumber.__onChangeText__ = text => {
      this._inputCvv._setCvv4((0, _CvvUtils.isCvv4Length)(text));
    };
  }
  render() {
    return /*#__PURE__*/_react.default.createElement(_reactNative.View, {
      style: this.props.containerStyle
    }, this.props.children);
  }
  _pullInput = (input, component) => {
    const instance = this.props[input]();
    if (!instance) {
      throw new Error('Missed result value for "' + input + '"');
    }
    const instancePrivate = instance;
    if (!instancePrivate._selfName) {
      throw new Error('Invalid component for "' + input + '"');
    }
    let selfName = instancePrivate._selfName();
    if (!selfName) {
      throw new Error('Missed result value for "' + input + '"');
    }
    let componentName = component.getInputName();
    if (componentName !== selfName) {
      throw new Error('Unexpected component "' + selfName + '" was set at "' + componentName + '" place');
    }
    return instancePrivate;
  };
}
exports.CardLayout = CardLayout;
//# sourceMappingURL=CardLayout.js.map