"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.CloudipspWebView = void 0;
var _react = _interopRequireDefault(require("react"));
var _reactNative = require("react-native");
var _reactNativeWebview = require("react-native-webview");
var _models = require("./models");
var _Native = require("./Native");
function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }
const addViewportMeta = `(${String(() => {
  // @ts-ignore
  const meta = document.createElement('meta');
  meta.setAttribute('content', 'width=device-width, user-scalable=0,');
  meta.setAttribute('name', 'viewport');
  // @ts-ignore
  const elementHead = document.getElementsByTagName('head');
  if (elementHead) {
    elementHead[0].appendChild(meta);
  } else {
    // @ts-ignore
    const head = document.createElement('head');
    head.appendChild(meta);
  }
})})();`;
class CloudipspWebView extends _react.default.Component {
  state = {
    baseUrl: undefined
  };
  _urlStartPattern = 'http://secure-redirect.cloudipsp.com/submit/#';
  _webViewRef = /*#__PURE__*/_react.default.createRef();
  __confirm__ = (baseUrl, html, cookies, apiHost, callbackUrl) => {
    if (this._onSuccess) {
      throw new Error('CloudipspWebView already waiting for confirmation');
    }
    if (cookies && _reactNative.Platform.OS === 'android') {
      _Native.Native.addCookies(baseUrl, cookies);
    }
    this.setState({
      baseUrl,
      html,
      cookies,
      apiHost,
      callbackUrl
    });
    return new Promise((resolve, reject) => {
      this._onSuccess = resolve;
      this._onFailure = reject;
    });
  };
  _onLoadStart = event => {
    if (!this._onSuccess) {
      return;
    }
    if (!this.state.baseUrl) {
      return;
    }
    const {
      apiHost,
      callbackUrl
    } = this.state;
    const url = event.nativeEvent.url;
    const detectsStartPattern = url.startsWith(this._urlStartPattern);
    let detectsCallbackUrl = false;
    let detectsApiToken = false;
    if (!detectsStartPattern) {
      detectsCallbackUrl = url.startsWith(callbackUrl);
      if (!detectsCallbackUrl) {
        detectsApiToken = url.startsWith(`${apiHost}/api/checkout?token=`);
      }
    }
    if (detectsStartPattern || detectsCallbackUrl || detectsApiToken) {
      var _this$_webViewRef$cur;
      let receipt;
      if (detectsStartPattern) {
        let jsonOfConfirmation = url.split(this._urlStartPattern)[1];
        let response;
        try {
          response = JSON.parse(jsonOfConfirmation);
        } catch (e) {
          response = JSON.parse(decodeURIComponent(jsonOfConfirmation));
        }
        receipt = _models.Receipt.__fromOrderData__(response.params);
      }
      this.setState({
        baseUrl: undefined,
        html: undefined,
        cookies: undefined
      }, () => {
        this._onSuccess(receipt);
        this._onSuccess = undefined;
      });
      (_this$_webViewRef$cur = this._webViewRef.current) === null || _this$_webViewRef$cur === void 0 ? void 0 : _this$_webViewRef$cur.goBack();
    }
  };
  render() {
    if (this.state.baseUrl === undefined) {
      return /*#__PURE__*/_react.default.createElement(_reactNative.View, null);
    } else {
      const {
        baseUrl,
        html
      } = this.state;
      return /*#__PURE__*/_react.default.createElement(_reactNativeWebview.WebView, {
        style: {
          flex: 1
        },
        ref: this._webViewRef,
        javaScriptEnabled: true,
        domStorageEnabled: true,
        scalesPageToFit: true,
        source: {
          baseUrl,
          html
        },
        injectedJavaScript: addViewportMeta,
        onLoadStart: this._onLoadStart
      });
    }
  }
}
exports.CloudipspWebView = CloudipspWebView;
//# sourceMappingURL=CloudipspWebview.js.map