import { Platform } from 'react-native';
import RNWebView from 'react-native-webview';
import { Failure, Receipt, req } from './models';
import { Native } from './Native';
export class Cloudipsp {
  __baseUrl__ = 'https://api.fondy.eu';
  __callbackUrl__ = 'http://callback';
  constructor() {
    let merchantId = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : req('merchantId');
    let cloudipspView = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : req('cloudipspView');
    this.__merchantId__ = merchantId;
    this.__cloudipspView__ = cloudipspView;
    if (!RNWebView) {
      throw new Error('"react-native-webview" module required');
    }
  }
  static supportsApplePay() {
    if (Platform.OS === 'ios') {
      return Native.supportsApplePay();
    } else {
      return Promise.resolve(false);
    }
  }
  static supportsGooglePay() {
    if (Platform.OS === 'android') {
      return Native.supportsGooglePay();
    } else {
      return Promise.resolve(false);
    }
  }
  pay() {
    let card = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : req('card');
    let order = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : req('order');
    if (!card.isValidCard()) {
      throw new Error('Card is not valid');
    }
    return this.__getToken__(order).then(token => {
      return this.__checkout__(token, card, order.email).then(checkout => this.__payContinue__(checkout, token, this.__callbackUrl__));
    });
  }
  payToken() {
    let card = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : req('card');
    let token = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : req('token');
    if (!card.isValidCard()) {
      throw new Error('Card is not valid');
    }
    let callbackUrl;
    return this.__getCallbackUrl__(token).then(_callbackUrl => {
      callbackUrl = _callbackUrl;
      return this.__checkout__(token, card, undefined);
    }).then(checkout => this.__payContinue__(checkout, token, callbackUrl));
  }
  static __assertApplePay__() {
    if (Platform.OS !== 'ios') {
      return Promise.reject(new Error('ApplePay available only for iOS'));
    }
  }
  applePayToken() {
    let token = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : req('token');
    Cloudipsp.__assertApplePay__();
    let config, applePayInfo, receiptFromToken, receiptFinal;
    return this.__order__(token).then(receipt => {
      receiptFromToken = receipt;
      return this.__getPaymentConfig__(null, null, token, 'https://apple.com/apple-pay', 'ApplePay');
    }).then(_config => {
      config = _config;
      return Native.applePay(config, receiptFromToken.amount, receiptFromToken.currency, ' ');
    }).then(_applePayInfo => {
      applePayInfo = _applePayInfo;
      return this.__checkoutApplePay__(token, receiptFromToken.email, config.payment_system, applePayInfo);
    }).then(checkout => this.__payContinue__(checkout, token, receiptFromToken.responseUrl)).then(receipt => {
      receiptFinal = receipt;
      return Native.applePayComplete(true);
    }).then(() => receiptFinal).catch(error => {
      Native.applePayComplete(false);
      throw error;
    });
  }
  applePay() {
    let order = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : req('order');
    Cloudipsp.__assertApplePay__();
    let config, applePayInfo, token, receipt;
    return this.__getPaymentConfig__(order.amount, order.currency, null, 'https://apple.com/apple-pay', 'ApplePay').then(_config => {
      config = _config;
      return Native.applePay(config, order.amount, order.currency, order.description);
    }).then(_applePayInfo => {
      applePayInfo = _applePayInfo;
      return this.__getToken__(order);
    }).then(_token => {
      token = _token;
      return this.__checkoutApplePay__(token, order.email, config.payment_system, applePayInfo);
    }).then(checkout => this.__payContinue__(checkout, token, this.__callbackUrl__)).then(_receipt => {
      receipt = _receipt;
      return Native.applePayComplete(true);
    }).then(() => receipt).catch(error => {
      Native.applePayComplete(false);
      throw error;
    });
  }
  static __assertGooglePay__() {
    if (Platform.OS !== 'android') {
      return Promise.reject(new Error('GooglePay available only for Android'));
    }
  }
  googlePayToken() {
    let token = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : req('token');
    Cloudipsp.__assertGooglePay__();
    let config, receiptFromToken;
    return this.__order__(token).then(receipt => {
      receiptFromToken = receipt;
      return this.__getPaymentConfig__(null, null, token, 'https://google.com/pay', 'GooglePay');
    }).then(_config => {
      config = _config;
      return Native.googlePay(config.data);
    }).then(googlePayInfo => {
      return this.__checkoutGooglePay__(token, receiptFromToken.email, config.payment_system, googlePayInfo);
    }).then(checkout => this.__payContinue__(checkout, token, receiptFromToken.responseUrl));
  }
  googlePay() {
    let order = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : req('order');
    Cloudipsp.__assertGooglePay__();
    let config, googlePayInfo, token;
    return this.__getPaymentConfig__(order.amount, order.currency, null, 'https://google.com/pay', 'GooglePay').then(_config => {
      config = _config;
      return Native.googlePay(config.data);
    }).then(_googlePayInfo => {
      googlePayInfo = _googlePayInfo;
      return this.__getToken__(order);
    }).then(_token => {
      token = _token;
      return this.__checkoutGooglePay__(token, order.email, config.payment_system, googlePayInfo);
    }).then(checkout => this.__payContinue__(checkout, token, this.__callbackUrl__));
  }
  __getPaymentConfig__(amount, currency, token, methodId, methodName) {
    const request = token ? {
      token
    } : {
      merchant_id: this.__merchantId__,
      currency,
      amount
    };
    return this.__callJson__('/api/checkout/ajax/mobile_pay', request).then(response => {
      if (response.error_message) {
        this.__handleResponseError__(response);
      }
      let data;
      for (let i = 0; i < response.methods.length; ++i) {
        const method = response.methods[i];
        if (method.supportedMethods === methodId) {
          data = method.data;
          break;
        }
      }
      if (!data) {
        if (token) {
          throw new Error(`${methodName} is not supported for token "${token}"`);
        } else {
          throw new Error(`${methodName} is not supported for merchant ${this.__merchantId__} and currency ${currency}`);
        }
      }
      const totalDetails = response.details.total;
      return {
        payment_system: response.payment_system,
        data,
        businessName: totalDetails.label
      };
    });
  }
  __getToken__(order) {
    let rqBody = {};
    rqBody.merchant_id = this.__merchantId__;
    rqBody.amount = String(order.amount);
    rqBody.currency = order.currency;
    rqBody.order_id = order.orderId;
    rqBody.order_desc = order.description;
    rqBody.email = order.email;
    const orderPrivate = order;
    if (orderPrivate._productId) {
      rqBody.product_id = orderPrivate._productId;
    }
    if (orderPrivate._paymentSystems) {
      rqBody.payment_systems = orderPrivate._paymentSystems;
    }
    if (orderPrivate._defaultPaymentSystem) {
      rqBody.default_payment_system = orderPrivate._defaultPaymentSystem;
    }
    if (orderPrivate._lifeTime) {
      rqBody.lifetime = orderPrivate._lifeTime;
    }
    if (orderPrivate._merchantData === undefined) {
      rqBody.merchant_data = '[]';
    } else {
      rqBody.merchant_data = orderPrivate._merchantData;
    }
    if (orderPrivate._version) {
      rqBody.version = orderPrivate._version;
    }
    if (orderPrivate._serverCallbackUrl) {
      rqBody.server_callback_url = orderPrivate._serverCallbackUrl;
    }
    if (orderPrivate._lang) {
      rqBody.lang = orderPrivate._lang.toString();
    }
    rqBody.preauth = orderPrivate._preAuth ? 'Y' : 'N';
    rqBody.required_rectoken = orderPrivate._requiredRecToken ? 'Y' : 'N';
    rqBody.verification = orderPrivate._verification ? 'Y' : 'N';
    if (orderPrivate._verificationType) {
      rqBody.verification_type = orderPrivate._verificationType.name;
    }
    rqBody = Object.assign(rqBody, orderPrivate._arguments);
    rqBody.response_url = this.__callbackUrl__;
    rqBody.delayed = orderPrivate._delayed ? 'Y' : 'N';
    return this.__apiCall__('/api/checkout/token', rqBody).then(response => response.token);
  }
  __checkout__(token, card, email) {
    const buildExp = (mm, yy) => {
      return (mm < 10 ? '0' : '') + mm + yy;
    };
    const cardPrivate = card;
    const rqBody = {
      card_number: cardPrivate.__getCardNumber__(),
      expiry_date: buildExp(cardPrivate.__getExpMm__(), cardPrivate.__getExpYy__()),
      token,
      email,
      payment_system: 'card'
    };
    if (card.getSource() === 'form') {
      rqBody.cvv2 = cardPrivate.__getCvv__();
    }
    return this.__apiCall__('/api/checkout/ajax', rqBody);
  }
  __checkoutApplePay__(token, email, paymentSystem, applePayData) {
    const rqBody = {
      token,
      email,
      payment_system: paymentSystem,
      data: applePayData
    };
    return this.__apiCall__('/api/checkout/ajax', rqBody);
  }
  __checkoutGooglePay__(token, email, paymentSystem, googlePayInfo) {
    const rqBody = {
      token,
      email,
      payment_system: paymentSystem,
      data: JSON.parse(googlePayInfo)
    };
    return this.__apiCall__('/api/checkout/ajax', rqBody);
  }
  __payContinue__(checkoutResponse, token, callbackUrl) {
    const getOrder = () => this.__order__(token);
    if (checkoutResponse.url.startsWith(callbackUrl)) {
      return getOrder();
    } else {
      return this.__url3ds__(checkoutResponse, callbackUrl).then(getOrder);
    }
  }
  __url3ds__(checkout, callbackUrl) {
    let body;
    let contentType;
    let sendData = checkout.send_data;
    if (sendData.PaReq === '') {
      body = JSON.stringify(sendData);
      contentType = 'application/json';
    } else {
      body = 'MD=' + encodeURIComponent(sendData.MD) + '&PaReq=' + encodeURIComponent(sendData.PaReq) + '&TermUrl=' + encodeURIComponent(sendData.TermUrl);
      contentType = 'application/x-www-form-urlencoded';
    }
    let cookies;
    return fetch(checkout.url, {
      method: 'POST',
      headers: {
        'Accept': 'application/json',
        'Content-Type': contentType,
        'User-Agent': 'React-Native'
      },
      body: body
    }).then(response => {
      cookies = response.headers.get('set-cookie');
      return response.text();
    }).then(html => {
      return this.__cloudipspView__(cloudipspView => {
        const cloudipspViewPrivate = cloudipspView;
        return cloudipspViewPrivate.__confirm__(checkout.url, html, cookies, this.__baseUrl__, callbackUrl);
      });
    });
  }
  __getCallbackUrl__(token) {
    return this.__apiCall__('/api/checkout/merchant/order', {
      token
    }).then(response => {
      return response.response_url;
    });
  }
  __order__(token) {
    return this.__apiCall__('/api/checkout/merchant/order', {
      token
    }).then(response => {
      return Receipt.__fromOrderData__(response.order_data, response.response_url);
    });
  }
  __apiCall__(path, request) {
    return this.__callJson__(path, request).then(response => {
      if (response.response_status === 'success') {
        return response;
      } else {
        this.__handleResponseError__(response);
      }
    });
  }
  __handleResponseError__(response) {
    throw new Failure(response.error_message, response.error_code, response.request_id);
  }
  __callJson__(path, request) {
    const url = this.__baseUrl__ + path;
    if (__DEV__) {
      console.log(`Request. ${url}`, request);
    }
    return fetch(url, {
      method: 'POST',
      headers: {
        'Accept': 'application/json',
        'Content-Type': 'application/json',
        'User-Agent': 'React-Native',
        'SDK-OS': Platform.OS,
        'SDK-Version': '1.0.0'
      },
      body: JSON.stringify({
        request
      })
    }).then(response => {
      return response.json();
    }).then(json => {
      if (__DEV__) {
        console.log(`Response. ${url}`, json);
      }
      return json.response;
    });
  }
}
//# sourceMappingURL=Cloudipsp.js.map