import { isCvv4Length } from '../CvvUtils';
function lunaCheck(cardNumber) {
  let sum = 0;
  let odd = true;
  for (let i = cardNumber.length - 1; i >= 0; i--) {
    try {
      let num = Number(cardNumber.charAt(i));
      odd = !odd;
      if (odd) {
        num *= 2;
      }
      if (num > 9) {
        num -= 9;
      }
      sum += num;
    } catch (e) {
      return false;
    }
  }
  return sum % 10 === 0;
}
function isValidExpireMonthValue(mm) {
  return mm >= 1 && mm <= 12;
}
function isValidExpireYearValue(yy) {
  return yy >= 23 && yy <= 99;
}
export class Card {
  getSource = () => {
    return 'form';
  };
  __getCardNumber__ = () => {
    throw new Error('Unimplemented');
  };
  __getExpYy__ = () => {
    throw new Error('Unimplemented');
  };
  __getExpMm__ = () => {
    throw new Error('Unimplemented');
  };
  __getCvv__ = () => {
    throw new Error('Unimplemented');
  };
  isValidCardNumber = () => {
    const cardNumber = this.__getCardNumber__();
    if (!(12 <= cardNumber.length && cardNumber.length <= 19)) {
      return false;
    }
    return lunaCheck(cardNumber);
  };
  isValidExpireMonth = () => {
    const mm = this.__getExpMm__();
    return isValidExpireMonthValue(mm);
  };
  isValidExpireYear = () => {
    const yy = this.__getExpYy__();
    if (!isValidExpireYearValue(yy)) {
      return false;
    }
    const year = new Date().getFullYear() - 2000;
    return year <= yy;
  };
  isValidExpireDate = () => {
    let mm = this.__getExpMm__();
    if (!isValidExpireMonthValue(mm)) {
      return false;
    }
    let yy = this.__getExpYy__();
    if (!isValidExpireYearValue(yy)) {
      return false;
    }
    const now = new Date();
    const year = now.getFullYear() - 2000;
    const month = now.getMonth() + 1;
    return yy > year || yy >= year && mm >= month;
  };
  isValidCvv = () => {
    let cvv = this.__getCvv__();
    if (isCvv4Length(this.__getCardNumber__())) {
      return cvv.length === 4;
    } else {
      return cvv.length === 3;
    }
  };
  isValidCard = () => {
    return this.isValidCardNumber() && this.isValidExpireDate() && this.isValidCvv();
  };
}
//# sourceMappingURL=Card.js.map